module.exports = (node, graph) => {
  const triggerIn = node.triggerIn('in')
  const triggerOut = node.triggerOut('out')
  const createBox = require('primitive-box')
  const computeEdges = require('geom-edges')
  const vec3 = require('pex-math').vec3
  const clamp = require('pex-math').utils.clamp
  const remap = require('pex-math').utils.map
  
  const geomIn = node.in('geomIn', null)
  const geomOut = node.out('geomOut', null)
  
  const positionIn = node.in('position', [0, 0, 0])
  const sizeIn = node.in('size', [1, 1, 1])
  const waveAmountIn = node.in('waveAmount', 0.2)
  const waveOffsetIn = node.in('waveOffset', 0.0)
  const showGizmoIn = node.in('showGizmo', true)
  
  const renderer = graph.renderer
  const ctx = graph.ctx
  
  const box = createBox()
  box.cells = computeEdges(box.cells)
  box.positions = box.positions.map((p) => vec3.add(p, [0, 0.5, 0]))
  box.primitive = ctx.Primitive.Lines
  
  const gizmoEnt = renderer.entity([
    renderer.material({
      baseColor: [1, 0, 0, 1],
      roughness: null,
      metallic: null
    }),
    renderer.geometry(box)
  ])
  
  sizeIn.onChange = () => {
    gizmoEnt.transform.set({ scale: sizeIn.value })
    update()
  }
  
  positionIn.onChange = () => {
    gizmoEnt.transform.set({ position: positionIn.value })
    update()
  }
  
  function update () {
    if (!geomIn.value) return
    
    var prevG = geomIn.value
    
    var g = {
      ...prevG
    }
    
    var position = positionIn.value
    var size = sizeIn.value
    var waveAmount = waveAmountIn.value
    var waveOffset = waveOffsetIn.value
    
    g.positions = g.positions.map((prevP) => {
      var p = vec3.copy(prevP)
      var tx = clamp(remap(p[0], position[0] - size[0] / 2, position[0] + size[0] / 2, 0, 1), 0, 1)    
      var tz = clamp(remap(p[2], position[2] - size[2] / 2, position[2] + size[2] / 2, 0, 1), 0, 1)    
      var h = (tx < 0.5) ? (tx * 2) : (1 - (tx - 0.5) * 2)// ^ shape
      h *= Math.sin(tz * Math.PI)
      p[1] += size[1] * h
      
      var waveScaleByDistance = 1 - Math.abs((tx - 0.5) * 2)
      var waveScaleByLength = Math.sin(tz * Math.PI)
      var wave = Math.sin((tz + waveOffset) * 2 * Math.PI)
      p[0] += waveAmount * waveScaleByDistance * waveScaleByLength * wave
      return p
    })
    
    geomOut.setValue(g)
  }
  
  geomIn.onChange = update
  waveAmountIn.onChange = update
  waveOffsetIn.onChange = update
  
  triggerIn.onTrigger = (props) => {
    if (showGizmoIn.value || node.selected) {
    	props.parentEntity.push([gizmoEnt])
    }
    triggerOut.trigger(props)
  }
  
  
  
};