module.exports = (node, graph) => {
  const triggerIn = node.triggerIn('in')
  const vec3 = require('pex-math').vec3
  const triggerOut = node.triggerOut('out')
  
  function makePrism (opts) {
    const r = opts.radius
    const position = opts.position || [0, 0, 0]
    const points = [
      [0, r, 0], [r, 0, 0],
      [0, -r, 0], [r, 0, 0],
  
      [0, r, 0], [-r, 0, 0],
      [0, -r, 0], [-r, 0, 0],
  
      [0, r, 0], [0, 0, r],
      [0, -r, 0], [0, 0, r],
  
      [0, r, 0], [0, 0, -r],
      [0, -r, 0], [0, 0, -r],
  
      [-r, 0, 0], [0, 0, -r],
      [r, 0, 0], [0, 0, -r],
      [r, 0, 0], [0, 0, r],
      [-r, 0, 0], [0, 0, r]
    ]
    points.forEach((p) => vec3.add(p, position))
    return points
  }
  
  function ambientLightGizmo () {
    return makePrism({ radius: 0.3 })
    .concat([
      [0, 0, -0.3], [0, 0, -1.3],
      [0, 0, 0.3], [0, 0, 1.3],
      [0.3, 0, 0], [1.3, 0, 0],
      [-0.3, 0, 0], [-1.3, 0, 0],
      [0, 0.3, 0], [0, 1.3, 0],
      [0, -0.3, 0], [0, -1.3, 0]
    ])
  }
  
  function directionalLightGizmo () {
    return makePrism({ radius: 0.3 })
    .concat([
      [0, 0, -0.3], [0, 0, -0.3 + 1],
      [0, 0, 0.3], [0, 0, 0.3 + 1],
      [0.3, 0, 0], [0.3, 0, 0 + 1],
      [-0.3, 0, 0], [-0.3, 0, 0 + 1],
      [0, 0.3, 0], [0, 0.3, 0 + 1],
      [0, -0.3, 0], [0, -0.3, 0 + 1]
    ])
  }
  
  function makeCircle (opts) {
    const points = []
  
    for (let i = 0; i < opts.steps; i++) {
      const t = i / opts.steps * 2 * Math.PI
      const x = Math.cos(t)
      const y = Math.sin(t)
      const pos = [0, 0, 0]
      pos[opts.axis ? opts.axis[0] : 0] = x
      pos[opts.axis ? opts.axis[1] : 1] = y
      vec3.scale(pos, opts.radius || 1)
      vec3.add(pos, opts.center || [0, 0, 0])
      points.push(pos)
    }
  
    const lines = points.reduce((lines, p, i) => {
      lines.push(p)
      lines.push(points[(i + 1) % points.length])
      return lines
    }, [])
  
    return lines
  }
  
  
  function pointLightGizmo (radius) {
    return makePrism({ radius: 0.3 })
    .concat(makeCircle({ center: [0, 0, 0], radius: radius, steps: 64, axis: [0, 1] }))
    .concat(makeCircle({ center: [0, 0, 0], radius: radius, steps: 64, axis: [0, 2] }))
    .concat(makeCircle({ center: [0, 0, 0], radius: radius, steps: 64, axis: [1, 2] }))
    .concat([
      [0, 0.3, 0], [0, 0.6, 0],
      [0, -0.3, 0], [0, -0.6, 0],
      [0.3, 0, 0], [0.6, 0, 0],
      [-0.3, 0, 0], [-0.6, 0, 0],
      [0, 0, 0.3], [0, 0, 0.6],
      [0, 0, -0.3], [0, 0, -0.6]
    ])
  }
  
  function spotLightGizmo (angle, range) {
    const r = range * Math.tan(angle)
  
    return makePrism({ radius: 0.3 })
    .concat([
      [0, 0, 0], [r, 0, range],
      [0, 0, 0], [-r, 0, range],
      [0, 0, 0], [0, r, range],
      [0, 0, 0], [0, -r, range]
    ])
    .concat(makeCircle({ radius: r, center: [0, 0, range], steps: 64, axis: [0, 1]}))
  }
  
  
  function areaLightGizmo (w, h) {
    const position =  [0, 0, 0]
    const points = [
      [-1, -1, 0], [1, -1, 0],
      [1, -1, 0], [1, 1, 0],
      [1, 1, 0], [-1, 1, 0],
      [-1, 1, 0], [-1, -1, 0],
      [-1, -1, 0], [1, 1, 0],
      [-1, 1, 0], [1, -1, 0],
  
      [-1, -1, 0], [-1, -1, 1 / 2],
      [1, -1, 0], [1, -1, 1 / 2],
      [1, 1, 0], [1, 1, 1 / 2],
      [-1, 1, 0], [-1, 1, 1 / 2],
      [0, 0, 0], [0, 0, 1 / 2]
    ]
    points.forEach((p) => {
      p[0] *= w / 2
      p[1] *= h / 2
      vec3.add(p, position)
    })
    return points
  }
  
  function perspectiveCameraGizmo (camera) {
    //return [[0, 0, 0], [0, 0, 1]]
    var points = []
    
    var position = [0, 0, 0]
  
    var frustumNear = -camera.near
    var frustumFar = -camera.far
    var frustumTop = Math.tan(camera.fov / 2) * frustumNear
    var frustumRight = frustumTop * camera.aspect
    var frustumLeft = -frustumTop;
    var frustumBottom = -frustumLeft;
  
    
    var front = [0, 0, 1]
    var up = [0, 1, 0]
    var right = [1, 0, 0]
  
    var frustumNearTopLeft     = vec3.addScaled(vec3.addScaled(vec3.addScaled([0, 0, 0], right, -frustumRight), up,  frustumTop), front, frustumNear)
    var frustumNearTopRight    = vec3.addScaled(vec3.addScaled(vec3.addScaled([0, 0, 0], right,  frustumRight), up,  frustumTop), front, frustumNear)
    var frustumNearBottomRight = vec3.addScaled(vec3.addScaled(vec3.addScaled([0, 0, 0], right,  frustumRight), up, -frustumTop), front, frustumNear)
    var frustumNearBottomLeft  = vec3.addScaled(vec3.addScaled(vec3.addScaled([0, 0, 0], right, -frustumRight), up, -frustumTop), front, frustumNear)
  
    var farNearRatio = frustumFar / frustumNear;
  
    var frustumFarTopLeft     = vec3.scale(vec3.copy(frustumNearTopLeft), farNearRatio)
    var frustumFarTopRight    = vec3.scale(vec3.copy(frustumNearTopRight), farNearRatio)
    var frustumFarBottomRight = vec3.scale(vec3.copy(frustumNearBottomRight), farNearRatio)
    var frustumFarBottomLeft  = vec3.scale(vec3.copy(frustumNearBottomLeft), farNearRatio)
  
    points.push(position, frustumFarTopLeft)
    points.push(position, frustumFarTopRight)
    points.push(position, frustumFarBottomRight)
    points.push(position, frustumFarBottomLeft)
    
    points.push(frustumNearTopLeft, frustumNearTopRight)
    points.push(frustumNearTopRight, frustumNearBottomRight)
    points.push(frustumNearBottomRight, frustumNearBottomLeft)
    points.push(frustumNearBottomLeft, frustumNearTopLeft)
  
    points.push(frustumFarTopLeft, frustumFarTopRight)
    points.push(frustumFarTopRight, frustumFarBottomRight)
    points.push(frustumFarBottomRight, frustumFarBottomLeft)
    points.push(frustumFarBottomLeft, frustumFarTopLeft)
  
    return points
  }
  
  
  var moduleExports = {
    ambientLightGizmo: ambientLightGizmo,
    directionalLightGizmo: directionalLightGizmo,
    pointLightGizmo: pointLightGizmo,
    spotLightGizmo: spotLightGizmo,
    areaLightGizmo: areaLightGizmo,
    perspectiveCameraGizmo: perspectiveCameraGizmo
  }
  
  triggerIn.onTrigger = (props) => {
    const newProps = Object.assign({}, props, {
      pexHelpers: moduleExports
    })
    triggerOut.trigger(newProps)
  }
  
};